% SDC ,  calculates  criteria exist in literature for evaluation of search capability of a search design.
% 
%    [kl, ekl, sp, wsp]=SDC(D,k,fi,rho,v,la)
%
%      For a design D satisfies the  Srivastava's condition (Srivastava, 1975) SDC  computes the 
%      searching probability (SP), weighted searching probability (WSP),
%      Kullback-Leibler criterion (KL) and expected  Kullback-Leibler criterion (EKL).
%
%
% The INPUT arguments:
%
%      D design matrix with entries -1 or 1.
%      K is the parameter in Srivasta's condition(1975).
%      FI is an integer indicator, with 2= when searching done for K factorial effects among two factor interactions, 3= among two and 3 factor interactions
%         and 4= among two and 3,4 factor interactions.
%      RHO is a K by 1 vector.
%      (V,LA) are pasitive parameters of Gamma(v,la) distribution using in calculating weighted searching probability.
% 
% Reference
%        Srivastava, J. N., (1975), Designs for searching non-negligible effects. In: Srivastava, J.N. (Ed.), A Survey of Statistical Design and Linear Models.
%        North-Holland, Elsevier, Amsterdam, pp. 507-519.



function  [kl, ekl, sp, wsp]=SDC(D,k,fi,rho,v,la)   
    error(nargchk(4,6,nargin));
    error(nargchk(0,4,nargout));
    [D_rows,D_cols] = size(D);
    for r=1:D_rows
        for s=1:D_cols
            if (D(r,s)~=1 && D(r,s)~=-1)
               error('SDC:BadDmatrix','The entries of matirx D must be -1 or 1');
            end
        end
    end   
    if k < 1
       error('SDC:BadK','K  must be an integer greater than 0.');
    end     
    if  (fi ~=2 && fi ~=3 && fi ~=4)
        error('SDC:BadFI',  'FI must be one of the values 2, 3 or 4');
    end    
    if  (fi==2)
        X2=prud(D,2);  %Matrix of two factor interactions
    elseif  (fi==3)
        X2=[prud(D,2)  prud(D,3)];  %Matrix of two and three factor interactions
    elseif  (fi==4)
        X2=[prud(D,2) prud(D,3) prud(D,4)];  %Matrix of two, three and four factor interactions
    end
    [m_rows,m_cols] = size(X2);
     if 2*k>m_cols
        error('SDC:BadK, K  must be an integer between 0 and %d', fix(m_cols/2) );
     end    
    if  k==1    && (nargin==4 || nargin==5)
           if nargout>3
              error('Too many output arguments');
           else
           sp=SP(D,rho,fi);
           kl=KL(D,k,rho,fi);
           ekl=EKL(D,k,fi);
           end
    elseif  k==1  && nargin==6
           sp =SP(D,rho,fi);
           wsp=WSP(D,fi,v,la);
           kl=KL(D,k,rho,fi);
           ekl=EKL(D,k,fi);           
    elseif  k>1 
           if nargout>2
              error('Too many output arguments');
           elseif nargin>4
               error('Too many input arguments');
           else
              kl=KL(D,k,rho,fi);
              ekl=EKL(D,k,fi); 
           end
    else           
    end
end
  
%--------------------------------------------------------------------------
%This function calculate Searching Probability Matrix for matrix D
%then calculate  SP.

function Sp=SP(D,rho,fi)
    [D_rows,D_cols] = size(D);
    for r=1:D_rows
       for s=1:D_cols
           if (D(r,s)~=1 && D(r,s)~=-1)
              error('KL:BadDmatrix','The entries of matirx D must be -1 or 1');
           end
       end
    end   
    if  (fi ~=2 && fi ~=3 && fi ~=4)
        error('KL:BadFI',  'FI must be one of the values 2, 3 or 4');
    end
    if (fi==2)        
       X2=prud(D,2);  %Matrix of two factor interactions
    elseif  (fi==3)
       X2=[prud(D,2)  prud(D,3)];  %Matrix of two & three factor interactions
    elseif  (fi==4)
       X2=[prud(D,2) prud(D,3) prud(D,4)]; %Matrix of two, three and four factor interactions
    end
    d2=size(X2);
    X1=[diag(ones(d2(1))) D];
    Q1=X1/(X1'*X1)*X1';
    dq=size(Q1);
    I=diag(diag(ones(dq(1))));
    Iq=I-Q1;
    rr=zeros(d2(2),1);
    b=zeros(d2(1),d2(2));
    x= zeros(d2(2),d2(2));
    c1=zeros(d2(2),d2(2));
    c2=zeros(d2(2),d2(2));
    sp=zeros(d2(2),d2(2));
    for i =1:d2(2)
        rr(i,1)=X2(:,i)'*Iq*X2(:,i);
        b(:,i)=Iq*X2(:,i)/(sqrt(rr(i,:)));
    end
    for i =1:d2(2)    
        for j= 1:d2(2)
            x(i,j)=b(:,i)'*b(:,j);
        end
    end
    for i =1:d2(2)
        c1(:,i)=sqrt(0.5*rr(i,:)*abs((1-x(:,i))));
        c2(:,i)=sqrt(0.5*rr(i,:)*(1+x(:,i)));
    end
    for i=1:d2(2)
        sp(:,i)=1-normcdf(c1(:,i).*rho)-normcdf(c2(:,i).*rho)+ 2*normcdf(c1(:,i).*rho).*normcdf(c2(:,i).*rho);
    end
    [rows,cols] = size(sp);
    spm=ones(rows,cols);
    for k = 1:rows
        for l = 1:cols
            if k~=l
               spm(k,l)=sp(k,l);  
            end
        end
       
    end
    Sp=min(min(spm)); %Searching probability (Shirakura etl. 1996)
end
 
%--------------------------------------------------------------------------
%This function calculate K-L for matrix D
function KL=KL(D,k,rho,fi)
      [D_rows,D_cols] = size(D);
      for r=1:D_rows
          for s=1:D_cols
            if (D(r,s)~=1 && D(r,s)~=-1)
                error('KL:BadDmatrix','The entries of matirx D must be -1 or 1');
            end
          end
      end
      if k < 1
        error('KL:BadK','K  must be an integer greater than 0.');
     end
     if length(rho)~=k
        error('KL:BadKorBadRHO','The length of vector RHO must be equal to K');
     end   
     if  (fi ~=2 && fi ~=3 && fi ~=4)
        error('KL:BadFI',  'FI must be one of the values 2, 3 or 4');
    end
    if (fi==2)
        X2=prud(D,2);  %Matrix of two factor interactions
    elseif  (fi==3)
        X2=[prud(D,2)  prud(D,3)];  %Matrix of two and three factor interactions
    elseif  (fi==4)
        X2=[prud(D,2) prud(D,3) prud(D,4)];  %Matrix of two, three and four factor interactions
    end
    d2=size(X2);
    %nc = 1:d2(2);
    %combc = combntns(nc,k);
    combc=combinator(d2(2),k,'c');
    combc_size = size(combc);
    combc_rows = combc_size(1);
    m=col(X2,k);
    c=zeros(combc_rows,combc_rows);
    m2=zeros(combc_rows,1);
    for i=1:combc_rows
         p1=m(:,(((i-1)*k)+1):(i*k));
        for j=1:combc_rows 
            if (i~=j)
               m1=m(:,(((j-1)*k)+1):(j*k));
               m2(j,1)=rho'*p1'*(eye(d2(1)) - m1/(m1'*m1)*m1')*p1*rho;
            else 
               m2(j,1)=5000;
           end
           
        end
        c(:,i)=m2;      
    end
   KL= min(min(c));
end

%--------------------------------------------------------------------------

%This function calculate K-L for matrix D.
   
function EKL=EKL(D,k,fi)
    [D_rows,D_cols] = size(D);
    for r=1:D_rows
        for s=1:D_cols
            if (D(r,s)~=1 && D(r,s)~=-1)
                error('KL:BadDmatrix','The entries of matirx D must be -1 or 1');
            end
        end
    end   
    if k < 1
        error('KL:BadK','K  must be an integer greater than 0.');
    end
    if  (fi ~=2 && fi ~=3 && fi ~=4)
        error('KL:BadFI',  'FI must be one of the values 2, 3 or 4');
    end
    if (fi==2)
        X2=prud(D,2); %Matrix of two factor interactions
    elseif  (fi==3)
        X2=[prud(D,2)  prud(D,3)]; %Matrix of two and three factor interactions
    elseif  (fi==4)
        X2=[prud(D,2) prud(D,3) prud(D,4)]; %Matrix of two, three and four factor interactions
    end
    d2=size(X2);
    combc = combinator(d2(2),k,'c');
    combc_size = size(combc);
    combc_rows = combc_size(1);
    m=col(X2,k);
    c=zeros(combc_rows,combc_rows);
    m2=zeros(combc_rows,1);
    for i=1:combc_rows
         p1=m(:,(((i-1)*k)+1):(i*k));
        for j=1:combc_rows 
            if (i~=j)
               m1=m(:,(((j-1)*k)+1):(j*k));
               m2(j,1)=trace(p1'*(eye(d2(1)) - m1/(m1'*m1)*m1')*p1);
           else 
               m2(j,1)=5000;
           end
           
        end
       c(:,i)=m2;      
    end
     EKL=min(min(c));
end


%--------------------------------------------------------------------------

%This function calculate weighted Searching Probability Matrix ,when we use gamma(v,la) as weight.

function WSP=WSP(D,fi,v,la)
    [D_rows,D_cols] = size(D);
    for r=1:D_rows
        for s=1:D_cols
            if (D(r,s)~=1 && D(r,s)~=-1)
                error('KL:BadDmatrix','The entries of matirx D must be -1 or 1');
            end
        end
    end   
    if (v<0 || la<0)
       error('KL:BadVLA','V and La  must be an integer greater than 0.');
    end     
    if  (fi ~=2 && fi ~=3 && fi ~=4)
        error('KL:BadFI',  'FI must be one of the values 2, 3 or 4');
    end
    if (fi==2)
        X2=prud(D,2);  %Matrix of two factor interactions
    elseif  (fi==3)
        X2=[prud(D,2)  prud(D,3)]; %Matrix of two and three factor interactions
    elseif  (fi==4)
        X2=[prud(D,2) prud(D,3) prud(D,4)]; %Matrix of two, three and four factor interactions
    end
    d2=size(X2);
    X1=[diag(ones(d2(1))) D];
    Q1=X1/(X1'*X1)*X1';
    dq=size(Q1);
    I=diag(diag(ones(dq(1))));
    Iq=I-Q1;
    rr=zeros(d2(2),1);
    b=zeros(d2(1),d2(2));
    x= zeros(d2(2),d2(2));
    c1=zeros(d2(2),d2(2));
    c2=zeros(d2(2),d2(2));
    wsp=zeros(d2(2),d2(2));
    for i =1:d2(2)
        rr(i,1)=X2(:,i)'*Iq*X2(:,i);
        b(:,i)=Iq*X2(:,i)/(sqrt(rr(i,:)));
    end
    for i =1:d2(2)
        for j= 1:d2(2)
            x(i,j)=b(:,i)'*b(:,j);
        end
            
    end
    for i =1:d2(2)
        for j=1:d2(2)
            if i~=j
               c1(j,i)=sqrt(0.5*rr(i,:)*(1-x(j,i)));
            end
        end
        c2(:,i)=sqrt(0.5*rr(i,:)*(1+x(:,i)));
     end
     
     for i=1:d2(2)
         wsp(:,i)=1-tcdf(c1(:,i).*(sqrt(v/la)),2*v)-tcdf(c2(:,i).*(sqrt(v/la)),2*v)+ 2*tcdf(c1(:,i).*(sqrt(v/la)),2*v).*tcdf(c2(:,i).*(sqrt(v/la)),2*v);
     end
         m_size = size(wsp);
         rows = m_size(1);
         cols = m_size(2);
         wspm=ones(rows,cols);
     for k = 1:rows
         for l = 1:cols
             if k~=l
                wspm(k,l)=wsp(k,l);  
             end
         end
     end
     WSP=min(min(wspm));%Weighted searching probability (Talebi and Esmailzadeh, 2011)
     
end


%--------------------------------------------------------------------------
%This function take matrix m and gives production of all combinations of
%size n of columns of m.
function out2 = prud(m, n)
    [m_rows,m_cols] = size(m);
    combs =combinator(m_cols,n,'c');
    combs_size = size(combs);
    combs_rows = combs_size(1);
    out = zeros(m_rows, n);
    out2 = zeros(m_rows,combs_rows);
    for i = 1:combs_rows
        out(:,1:n) =  m(:,combs(i,:)) ;
        b1=ones(m_rows);
        b=b1(:,1);
        for j =1:n
            b=b.*out(:,j);
        end
        out2(:,i)=b;
    end

end
%--------------------------------------------------------------------------
%This function gives all combinations of size n of column of the matrix m 
function out=col(m, n)
    m_size = size(m);
    m_rows = m_size(1);
    m_cols = m_size(2);    
    combs = combinator(m_cols,n,'c');
    combs_size = size(combs);
    combs_rows = combs_size(1);
    out = zeros(m_rows,n*combs_rows);
        for i = 1:combs_rows
            out(:,(((i-1)*n)+1):(i*n)) = m(:,combs(i,:));
        end
      
     
end

