%SPM calculates search probability matrix given in Ghosh and Teschmacher (2002) 
%
%SPM(D,rho,fi,fname) 
%        calculate the seaching probability matrix for a given search design D at given value of rho. Then save it in a .txt file. 
%Input arguments
%
%       D design matrix with entries -1 or 1.
%       RHO is a positive real number.
%       FI is an integer indicator, with 2= when searching done for K factorial effects among two factor interactions, 3= among two and 3 factor interactions
%          and 4= among two and 3,4 factor interactions.
%fname is a character output file name.
%
%Reference
%       S. Ghosh and L. Teschmacher,(2002). Comparisons of search designs using search
%       probabilities, J. Statist. Plann. Inference, 104,439-458.
%

function SPM(D,rho,fi,fname)
    fh = fopen(fname, 'w'); % file handler    
    [D_rows,D_cols] = size(D);
    for r=1:D_rows
       for s=1:D_cols
           if (D(r,s)~=1 && D(r,s)~=-1)
              error('KL:BadDmatrix','The entries of matirx D must be -1 or 1');
           end
       end
    end   
    if  (fi ~=2 && fi ~=3 && fi ~=4)
        error('KL:BadFI',  'FI must be one of the values 2, 3 or 4');
    end
    if (fi==2)        
       X2=prud(D,2);  %Matrix of two factor interactions
    elseif  (fi==3)
       X2=[prud(D,2)  prud(D,3)];  %Matrix of two & three factor interactions
    elseif  (fi==4)
       X2=[prud(D,2) prud(D,3) prud(D,4)]; %Matrix of two, three and four factor interactions
    end
    d2=size(X2);
    X1=[diag(ones(d2(1))) D];
    Q1=X1/(X1'*X1)*X1';
    dq=size(Q1);
    I=diag(diag(ones(dq(1))));
    Iq=I-Q1;
    rr=zeros(d2(2),1);
    b=zeros(d2(1),d2(2));
    x= zeros(d2(2),d2(2));
    c1=zeros(d2(2),d2(2));
    c2=zeros(d2(2),d2(2));
    c=zeros(d2(2),d2(2));
    sp=zeros(d2(2),d2(2));
    for i =1:d2(2)
        rr(i,1)=X2(:,i)'*Iq*X2(:,i);
        b(:,i)=Iq*X2(:,i)/(sqrt(rr(i,:)));
    end
    for i =1:d2(2)    
        for j= 1:d2(2)
            x(i,j)=b(:,i)'*b(:,j);
        end
    end
    for i =1:d2(2)
        c1(:,i)=sqrt(0.5*rr(i,:)*abs((1-x(:,i))));
        c2(:,i)=sqrt(0.5*rr(i,:)*(1+x(:,i)));
        c(:,i)=sqrt(0.5*rr(i,:)*abs((1-abs(x(:,i)))));
    end
    for i=1:d2(2)
        sp(:,i)=1-normcdf(c1(:,i).*rho)-normcdf(c2(:,i).*rho)+ 2*normcdf(c1(:,i).*rho).*normcdf(c2(:,i).*rho);
    end
    [rows,cols] = size(sp);
    spm=zeros(rows,cols);
    for k = 1:rows
        for l = 1:cols
            if k~=l
               spm(k,l)=sp(k,l);
            end
        end
       
    end
    fprintf(fh, 'Search Probability Matrix at rho = %.2f\n%d \n',rho);
    prn_mat(fh, spm);
    fprintf(fh, '\n');
    fprintf(fh, 'C Matrix  \n');
    prn_mat(fh, c);
    fprintf(fh, '\n');
    fprintf(fh, 'r vector  \n');
    prn_mat(fh, rr');
    fclose(fh);
end
%--------------------------------------------------------------------------
%This function take matrix m and gives production of all combinations of
%size n of columns of m.
function out2 = prud(m, n)
    [m_rows,m_cols] = size(m);
    %ns = 1:m_cols;
    combs = combinator(m_cols,n,'c');%combntns(ns,n);
    combs_size = size(combs);
    combs_rows = combs_size(1);
    out = zeros(m_rows, n);
    out2 = zeros(m_rows,combs_rows);
    for i = 1:combs_rows
        out(:,1:n) =  m(:,combs(i,:)) ;
        b1=ones(m_rows);
        b=b1(:,1);
        for j =1:n
            b=b.*out(:,j);
        end
        out2(:,i)=b;
    end

end

%===========================================================================
function prn_mat(fh, m)
    fprintf(fh, '--------------------------------------------------------------\n\n');
    m_size = size(m);
    rows = m_size(1);
    cols = m_size(2);
    for i = 1:rows
        for j = 1:cols
            fprintf(fh, '%.5f\t', m(i,j));            
        end
        fprintf(fh, '\n');
    end
end